# example8.c
#
# This example illustrates how to use remote procedure calls and how
# to process return parameter values from stored procedures.
#
# The example uses the following stored procedure, named "rpctest",
# which it assumes is located in the user's default database.  Before
# running this example, you must create "rpctest" in your default
# database.
#
#     create procedure rpctest
#         (@param1 int out,
#          @param2 int out,
#          @param3 int out,
#          @param4 int)
#     as
#     begin
#         select "rpctest is running."
#         select @param1 = 11
#         select @param2 = 22
#         select @param3 = 33
#         select @param1
#
#         return 123
#     end
import sys
from mssqldb import *

import exutils

def doexit(s):
    print(s)
    dbexit()
    sys.exit(1)

print "Demo of SQL queries in a command batch\n"

# Initialize DB-Library. */
if not dbinit():
    sys.exit(1)

# Install the user-supplied error-handling and message-handling
# routines. They are defined at the bottom of this source file.
dberrhandle(exutils.err_handler)
dbmsghandle(exutils.msg_handler)

# Allocate and initialize the LOGINREC structure to be used to open a
# connection to SQL Server.
login = dblogin()
login.dbsetluser(exutils.USER)
login.dbsetlpwd(exutils.PASSWORD)
login.dbsetlapp("rpcexample")

dbproc = login.dbopen(exutils.SERVER)
dbproc.dbuse('test')

param1 = DataBuf(SQLINT4)
param1.set(1)
param2 = DataBuf(SQLINT4)
param2.set(2)
param3 = DataBuf(SQLINT4)
param3.set(3)
param4 = DataBuf(SQLINT4)
param4.set(4)

# Make the rpc.
if dbproc.dbrpcinit("rpctest") == FAIL:
    doexit("dbrpcinit failed.\n")

if dbproc.dbrpcparam("@param1", param1, DBRPCRETURN) == FAIL:
    doexit("dbrpcparam 1 failed.\n")

if dbproc.dbrpcparam("@param2", param2, DBRPCRETURN) == FAIL:
    doexit("dbrpcparam 2 failed.\n")

if dbproc.dbrpcparam("@param3", param3, DBRPCRETURN) == FAIL:
    doexit("dbrpcparam 3 failed.\n")

if dbproc.dbrpcparam("@param4", param4) == FAIL:
    doexit("dbrpcparam 4 failed.\n")

if dbproc.dbrpcsend() == FAIL:
    doexit("dbrpcsend failed.\n")

if dbproc.dbsqlok() == FAIL:
    doexit("dbsqlok failed.\n")

while 1:
    return_code = dbproc.dbresults()
    if return_code == NO_MORE_RESULTS:
        break
    if return_code == FAIL:
        doexit("dbresults failed.\n")
    # Print any rows that may have been returned.
    dbproc.dbprrow()

# Examine any return parameters that may have arrived.
numrets = dbproc.dbnumrets()
print "%d return values received.\n" % numrets

FMTSTR = "%-8.8s %-8.8s %-8.8s %-8.8s"
FMTSTR1 = "%-8.8s %-8.8s %8.8d %8.8d"

if numrets > 0:
    print FMTSTR % ("Name", "Type", "Length", "Value")
    print FMTSTR % ("------------", "------------",
                    "------------", "------------")
val = DataBuf(SQLINT4)
for i in range(1, numrets + 1):
    dbproc.dbretdata(val, i)
    print FMTSTR1 % (dbproc.dbretname(i),
                     dbprtype(dbproc.dbrettype(i)),
                     dbproc.dbretlen(i),
                     val.get())

if dbproc.dbhasretstat():
    print "Return status = %d" % dbproc.dbretstatus()
else:
    print 'No return status'

dbexit()
