/******************************************************************

 Copyright 2001 by Object Craft P/L, Melbourne, Australia.

 LICENCE - see LICENCE file distributed with this software for details.

******************************************************************/

#include "mssqldb.h"

PyTypeObject MoneyType;

static struct PyMethodDef Money_methods[] = {
    { NULL }			/* sentinel */
};

int money_as_string(PyObject *obj, char *text)
{
    int len;

    len = dbconvert(NULL,
		    SQLMONEY, (BYTE*)&((MoneyObj*)obj)->money, -1,
		    SQLCHAR, text, MONEY_LEN);
    if (len > 0)
	text[len] = '\0';
    return len;
}

PyObject *money_alloc(DBMONEY *money)
{
    MoneyObj *self;

    self = PyObject_NEW(MoneyObj, &MoneyType);
    if (self == NULL)
	return NULL;

    memcpy(&self->money, money, sizeof(self->money));
    return (PyObject*)self;
}

PyObject *Money_FromString(PyObject *obj)
{
    DBMONEY money;
    char *str = PyString_AsString(obj);
    
    if (dbconvert(NULL,
		  SQLCHAR, str, -1,
		  SQLMONEY, (BYTE*)&money, -1) <= 0) {
	PyErr_SetString(PyExc_TypeError, "money from string conversion failed");
	return NULL;
    }
    return money_alloc(&money);
}

static void Money_dealloc(MoneyObj *self)
{
    PyMem_DEL(self);
}

static int Money_print(MoneyObj *self, FILE *fp, int flags)
{
    char text[MONEY_LEN];

    money_as_string((PyObject*)self, text);
    fputs(text, fp);
    return 0;
}

static PyObject *Money_repr(MoneyObj *self)
{
    char text[MONEY_LEN];

    money_as_string((PyObject*)self, text);
    return PyString_FromString(text);
}

static PyObject *Money_int(MoneyObj *v)
{
    DBINT int_value;

    if (dbconvert(NULL,
		  SQLMONEY, (BYTE*)&v->money, -1,
		  SQLINT4, (BYTE*)&int_value, -1) <= 0) {
	PyErr_SetString(PyExc_TypeError, "int conversion failed");
	return NULL;
    }
    return PyInt_FromLong(int_value);
}

static PyObject *Money_long(MoneyObj *v)
{
    char *end;
    char text[MONEY_LEN];

    money_as_string((PyObject*)v, text);
    return PyLong_FromString(text, &end, 10);
}

static PyObject *Money_float(MoneyObj *v)
{
    DBFLT8 float_value;

    if (dbconvert(NULL,
		  SQLMONEY, (BYTE*)&v->money, -1,
		  SQLFLT8, (BYTE*)&float_value, -1) <= 0) {
	PyErr_SetString(PyExc_TypeError, "float conversion failed");
	return NULL;
    }
    return PyFloat_FromDouble(float_value);
}

static PyNumberMethods Money_as_number = {
    (binaryfunc)0,		/*nb_add*/
    (binaryfunc)0,		/*nb_subtract*/
    (binaryfunc)0,		/*nb_multiply*/
    (binaryfunc)0,		/*nb_divide*/
    (binaryfunc)0,		/*nb_remainder*/
    (binaryfunc)0,		/*nb_divmod*/
    (ternaryfunc)0,		/*nb_power*/
    (unaryfunc)0,		/*nb_negative*/
    (unaryfunc)0,		/*nb_positive*/
    (unaryfunc)0,		/*nb_absolute*/
    (inquiry)0,			/*nb_nonzero*/
    (unaryfunc)0,		/*nb_invert*/
    (binaryfunc)0,		/*nb_lshift*/
    (binaryfunc)0,		/*nb_rshift*/
    (binaryfunc)0,		/*nb_and*/
    (binaryfunc)0,		/*nb_xor*/
    (binaryfunc)0,		/*nb_or*/
    (coercion)0,		/*nb_coerce*/
    (unaryfunc)Money_int,	/*nb_int*/
    (unaryfunc)Money_long,	/*nb_long*/
    (unaryfunc)Money_float,	/*nb_float*/
    (unaryfunc)0,		/*nb_oct*/
    (unaryfunc)0,		/*nb_hex*/
};

#define OFFSET(x) offsetof(MoneyObj, x)

static struct memberlist Money_memberlist[] = {
    { NULL }
};

static PyObject *Money_getattr(MoneyObj *self, char *name)
{
    PyObject *rv;

    rv = PyMember_Get((char*)self, Money_memberlist, name);
    if (rv)
	return rv;
    PyErr_Clear();
    return Py_FindMethod(Money_methods, (PyObject *)self, name);
}

static int Money_setattr(MoneyObj *self, char *name, PyObject *v)
{
    if (v == NULL) {
	PyErr_SetString(PyExc_AttributeError, "Cannot delete attribute");
	return -1;
    }
    return PyMember_Set((char*)self, Money_memberlist, name, v);
}

static char MoneyType__doc__[] = 
"";

PyTypeObject MoneyType = {
    PyObject_HEAD_INIT(0)
    0,				/*ob_size*/
    "MoneyType",		/*tp_name*/
    sizeof(MoneyObj),		/*tp_basicsize*/
    0,				/*tp_itemsize*/
    /* methods */
    (destructor)Money_dealloc,/*tp_dealloc*/
    (printfunc)Money_print,	/*tp_print*/
    (getattrfunc)Money_getattr, /*tp_getattr*/
    (setattrfunc)Money_setattr, /*tp_setattr*/
    (cmpfunc)0,			/*tp_compare*/
    (reprfunc)Money_repr,	/*tp_repr*/
    &Money_as_number,		/*tp_as_number*/
    0,				/*tp_as_sequence*/
    0,				/*tp_as_mapping*/
    (hashfunc)0,		/*tp_hash*/
    (ternaryfunc)0,		/*tp_call*/
    (reprfunc)0,		/*tp_str*/

    0L,0L,0L,0L,
    MoneyType__doc__
};

char money_new__doc__[] =
"money(s) -> Money\n"
"\n"
"Create a money object.";

/* Implement the mssqldb.money() method
 */
PyObject *MoneyType_new(PyObject *module, PyObject *args)
{
    char *str;
    DBMONEY money;

    if (!PyArg_ParseTuple(args, "s", &str))
	return NULL;

    if (dbconvert(NULL,
		  SQLCHAR, str, -1,
		  SQLMONEY, (BYTE*)&money, -1) <= 0) {
	PyErr_SetString(PyExc_TypeError, "money from string conversion failed");
	return NULL;
    }
    return money_alloc(&money);
}

/* Used in unpickler
 */
static PyObject *money_constructor = NULL;

/* Register the money type with the copy_reg module.  This allows
 * Python to (un)pickle money objects.  The equivalent Python code
 * is this:
 *
 * def pickle_money(n):
 *     return money, (str(n))
 * 
 * copy_reg.pickle(type(money(1)), pickle_money, money)
 */
char pickle_money__doc__[] =
"pickle_money(n) -> money, (str(n))\n"
"\n"
"Used to pickle the money data type.";

/* Money pickling function
 */
PyObject *pickle_money(PyObject *module, PyObject *args)
{
    MoneyObj *obj = NULL;
    PyObject *values = NULL,
	*tuple = NULL;
    char text[MONEY_LEN];

    if (!PyArg_ParseTuple(args, "O!", &MoneyType, &obj))
	goto error;
    money_as_string((PyObject*)obj, text);
    if ((values = Py_BuildValue("(s)", text)) == NULL)
	goto error;
    tuple = Py_BuildValue("(OO)", money_constructor, values);

error:
    Py_XDECREF(values);
    return tuple;
}

/* Register Money type pickler
 */
void copy_reg_money(PyObject *dict)
{
    PyObject *module = NULL,
	*pickle_func = NULL,
	*pickler = NULL,
	*obj = NULL;

    module = PyImport_ImportModule("copy_reg");
    if (module == NULL)
	goto error;
    if ((pickle_func = PyObject_GetAttrString(module, "pickle")) == NULL)
	goto error;
    if ((money_constructor = PyDict_GetItemString(dict, "money")) == NULL)
	goto error;
    if ((pickler = PyDict_GetItemString(dict, "pickle_money")) == NULL)
	goto error;
    Py_XINCREF(money_constructor);
    obj = PyObject_CallFunction(pickle_func, "OOO",
				&MoneyType, pickler, money_constructor);

error:
    Py_XDECREF(obj);
    Py_XDECREF(pickler);
    Py_XDECREF(pickle_func);
    Py_XDECREF(module);
}
